#ifndef _FICHA_H_
#define _FICHA_H_

#include "imagen.h"
#include "video.h"
/** Enumerado de los colores de las fichas. */
enum Color{VERDE = 0, AZUL, ROJO, NEGRO};

/** Clase Ficha. Clase para el manejo y funcionamiento de las fichas del juego.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Ficha {
 public:
  // Constructores

  /** Constructor de la clase Ficha.
   * @param com Indica si una determinada ficha es comodín o no. Por defecto será falso.
   * @param n Indica el valor numérico de la ficha. Por defecto será 1.
   * @param c Indica el color de la ficha. Por defecto será VERDE.
   */
  Ficha(bool com = false, const unsigned n = 1, const Color c = VERDE);

  /** Constructor de copia.
   * @param f Es una referencia constante a un objeto de la clase Ficha.
   */
  Ficha(const Ficha& f);

  // Op. asignación

  /** Operador de asignación.
   * @param f Es una referencia constante a un objeto de la clase Ficha.
   */
  Ficha& operator =(const Ficha& f);

  // Modificadores

  /** Método que modifica la posición de la imagen de la ficha en la pantalla. 
   * @param x Indica la nueva coordenada en el eje x.
   * @param y Indica la nueva coordenada en el eje y.
   */
  void posicionar(int x, int y);

  /** Método que modifica el valor interno del número y el color del comodín. 
   * @param n Indica el nuevo número del comodín.
   * @param c Indica el nuevo color del comodín.
   * @return Devuelve si la modificación se ha realizado correctamente o no.
   */
  bool modificarComodin(const int n, const Color c);

  // Observadores

  /** Método que devuelve el valor numérico de la ficha. 
   * @return Devuelve el número de la ficha.
   */
  const unsigned numero() const;

  /** Método que devuelve el color de la ficha. 
   * @return Devuelve el color de la ficha.
   */
  const Color color() const;

  /** Método que devuelve si la ficha es comodín o no. 
   * @return Devuelve si la ficha es comodín o no.
   */
  bool comodin() const;

  /** Método que devuelve si una ficha es nula (Si su valor numérico es 0). 
   * @return Devuelve si una ficha es nula.
   */
  bool fichaNula() const;

  /** Método que muestra la ficha por pantalla. 
   * @param v Puntero a la clase Video.
   */
  void mostrarFicha(Video *v);

  // Destructor

  /** Destructor de la clase Ficha
   */
  ~Ficha();

 private:
  bool fComodin;
  unsigned num;  
  Color col; 
  Imagen* imagen;

  /// Funciones privadas
  /** Método para obtener la ruta donde se encuentra la imagen de la ficha
   * @return Devuelve una cadena de caracteres constante con la ruta donde se encuentra la imagen de la ficha.
   */
  const char* obtenerRuta();
};


// Operaciones externas a la clase:

/** Función que comprueba si una ficha es menor que otra.
 * @param f1 Referencia constante a un objeto de la clase Ficha.
 * @param f2 Referencia constante a un objeto de la clase Ficha.
 * @return Devuelve si f1 es menor que f2.
 */
bool operator <(const Ficha& f1, const Ficha& f2);

/** Función que comprueba si dos fichas son iguales.
 * @param f1 Referencia constante a un objeto de la clase Ficha.
 * @param f2 Referencia constante a un objeto de la clase Ficha.
 * @return Devuelve si f1 es igual que f2.
 */
bool operator ==(const Ficha& f1, const Ficha& f2);

/** Función que comprueba si una ficha es distinta a otra.
 * @param f1 Referencia constante a un objeto de la clase Ficha.
 * @param f2 Referencia constante a un objeto de la clase Ficha.
 * @return Devuelve si f1 es distinta que f2.
 */
bool operator !=(const Ficha& f1, const Ficha& f2);


// Funciones en linea:
inline const unsigned Ficha::numero() const { return num; }
inline const Color Ficha::color() const { return col; }
inline bool Ficha::comodin() const { return fComodin; }
inline bool Ficha::fichaNula() const { return num == 0; }
inline void Ficha::mostrarFicha(Video *v) { imagen->dibujar(v); }
inline Ficha::~Ficha() {}

#endif

