#ifndef _GESTORPARTIDA_H_
#define _GESTORPARTIDA_H_

#include <string>
#include <vector>
#include "video.h"
#include "imagen.h"
#include "sonido.h"
#include "fuente.h"
#include "puntero.h"
#include "jugador.h"
#include "partida.h"

using namespace std;

/** Clase GestorPartida. Clase para gestionar los menús referentes a la partida.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class GestorPartida
{
 public:

 // Constructor

  /** Constructor de la clase GestorPartida.
   */
  GestorPartida();

  // Modificadores

  /** Método para jugar una partida simple contra el ordenador.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void partidaSimple(Video *v);

  /** Método para jugar un concurso contra otras personas.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void concurso(Video *v);

  /** Destructor de la clase GestorPartida.
   */
  ~GestorPartida();

 private:
  Imagen *fondo, *incrementar, *decrementar, *suena, *nosuena, *confirmar, *cancelar;
  Fuente textos, ronda;
  Musica musica;
  string nombre;
  unsigned numJugadores;
  Partida *partida;
  typedef vector <Jugador *> Jugadores;
  Jugadores jugadores;

  /** Método auxiliar para seleccionar el número de jugadores y los jugadores que participarán en el concurso.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void seleccionJugadores(Video *v);

  /** Método auxiliar para seleccionar el número de jugadores (una persona y 1, 2 o 3 robots) y el jugador que participará en la partida simple.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void seleccionJugadorSimple(Video *v);

  /** Método para mostrar la pantalla en seleccionJugador y seleccionJugadorSimple.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @param jugActual Indica el número de jugador que se está introduciendo en la partida.
   * @param zona Indica en que parte de la selección de los jugadores estamos, 1 si se está en la selección del número de jugadores y 2 si se está en la parte de la selección de jugadores.
   * @param nom Indica el nombre del jugador que se va a introducir en la partida.
   */
  void actualizarPantalla(Video* v, unsigned jugActual, int zona, string nom); 

  /** Método que devuelve si las coordenadas de un puntero están dentro del area de la flecha de incremento o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area de la flecha de incremento.
   */
  bool areaIncremento(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area de la flecha de decremento o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area de la flecha de decremento.
   */
  bool areaDecremento(const Puntero& p);

  /** Método que devuelve el caracter de la tecla que se pulsó en el evento en caso de ser válida o el caracter '+', si no lo es.
   * @param evento Evento de teclado que contiene la tecla que se pulsó.
   * @return Devuelve devuelve el caracter de la tecla que se pulsó en caso de ser válido o el caracter '+', si no lo es.
   */
  char caracter(SDL_Event evento);

  /** Método que devuelve si un determinado nombre de jugador está reservado para la partida o no.
   * @param nombre Es el nombre del cual se quiere comprobar su disponibilidad.
   * @return Devuelve si el nombre está reservado o no.
   */
  bool reservado(string nombre);

  /** Método que desordena el vector que contiene a los jugadores.
   */
  void desordenarJugadores();

  /** Método que incrementa la posición de los jugadores en el vector, el primero pasa a ser el segundo [...] y el último pasa a ser el primero (Necesario para el concurso).
   */
  void incrementarPosicion();

  /** Método que muestra la pantalla con el número de ronda actual, necesario para el concurso.
   * @param r Indica el número de ronda actual.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void pantallaRonda(const unsigned r, Video* v);
};

// Funciones externas a la clase

/** Función que devuelve si las coordenadas de un puntero están dentro del area de confirmación o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area de confirmación.
 */
bool areaConfirmar(const Puntero& p);

/** Función que devuelve si las coordenadas de un puntero están dentro del area de cancelación o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area de cancelación.
 */
bool areaCancelar(const Puntero& p);

inline GestorPartida::~GestorPartida() {}

#endif
