#ifndef _JUGADOR_H_
#define _JUGADOR_H_

#include <string>
#include "soporte.h"

using namespace std;

/** Clase Jugador. Clase para el manejo y la gestión  de los jugadores.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Jugador {
 public:

  /** Enumerado que contiene los distintos rangos que puede tener un jugador. */
  enum Rango{NOVEL = 1, MEDIO, AVANZADO, EXPERTO};

  // Constructor

  /** Constructor de la clase Jugador. Este constructor comprueba si el nombre del jugador está o no registrado en el sistema, si está lo carga, si no está lo crea y lo registra.
   * @param nombre Referencia constante de la clase string que contiene el nombre del jugador.
   * @param robt Indica si un determinado jugador será un robot o no. Por defecto es falso.
   */
  Jugador(const string& nombre, bool robt = false);

  // Modificadores

  /** Método para guardar los progresos de un jugador.
   */
  void guardarJugador();

  /** Método para borrar un jugador del sistema. Esta función borra su registro.
   */
  void borrarJugador();

  /** Método para añadir una partida ganada al jugador.
   */
  void partidaGanada();

  /** Método para añadir una partida perdida al jugador.
   */
  void partidaPerdida();

  /** Método para añadir una partida abandonada al jugador.
   */
  void partidaAbandonada();

  /** Método para actualizar los puntos del jugador. Si ha ganado, se restan los puntos de todos los demás jugadores, si ha perdido o abandonado se suman los puntos de su soporte.
   * @param puntos son los puntos que se van a sumar o restar.
   * @param situacion 1 si es el ganador, 0 si es perdedor, -1 si ha abandonado la partida.
   */
  void puntuacionPartida(int puntos, int situacion);

  // Métodos para la partida

  /** Método que inicializa el soporte de un jugador, robando 14 fichas del montón.
   * @param m Montón del que se extraen las fichas.
   */
  void iniciarSoporte(Monton& m);

  /** Método que devuelve el soporte del jugador para poder ser manipulado.
   * @return Devuelve el soporte del jugador.
   */
  Soporte& obtenerSoporte();

  // Observadores

  /** Método que devuelve el nombre del jugador.
   */
  const string nombre() const;

  /** Método que devuelve la puntuación de la partida del jugador.
   */
  const int puntuacionPartida() const;

  /** Método que devuelve la puntuación general del jugador.
   */
  const int puntuacionGeneral() const;

  /** Método que devuelve el número de partidas ganadas del jugador.
   */
  const int partidasGanadas() const;

  /** Método que devuelve el número de partidas perdidas del jugador.
   */
  const int partidasPerdidas() const;

  /** Método que devuelve el número de partidas abandonadas del jugador.
   */
  const int partidasAbandonadas() const;

  /** Método que devuelve el porcentaje de partidas ganadas del jugador.
   */
  const double porGanadas() const;

  /** Método que devuelve el porcentaje de partidas perdidas del jugador.
   */
  const double porPerdidas() const;

  /** Método que devuelve el porcentaje de partidas abandonadas del jugador.
   */
  const double porAbandonadas() const;

  /** Método que devuelve el rango del jugador.
   */
  const Rango rangoJugador() const;

  /** Método que devuelve si un jugador es robot o no.
   */
  bool jRobot() const;

 private:
  bool robot;
  string nom;
  int puntPartida;
  int puntGeneral, partGanadas, partPerdidas, partAbandonadas;
  double porcentajeGanadas, porcentajePerdidas, porcentajeAbandonadas;
  Jugador::Rango rango;
  Soporte soporte;
};

inline Soporte& Jugador::obtenerSoporte() { return soporte; }
inline const string Jugador::nombre() const { return nom; }
inline const int Jugador::puntuacionPartida() const { return puntPartida; }
inline const int Jugador::puntuacionGeneral() const { return puntGeneral; }
inline const int Jugador::partidasGanadas() const { return partGanadas; }
inline const int Jugador::partidasPerdidas() const { return partPerdidas; }
inline const int Jugador::partidasAbandonadas() const { return partAbandonadas; }
inline const double Jugador::porGanadas() const { return porcentajeGanadas; }
inline const double Jugador::porPerdidas() const { return porcentajePerdidas; }
inline const double Jugador::porAbandonadas() const { return porcentajeAbandonadas; }
inline const Jugador::Rango Jugador::rangoJugador() const { return rango; }
inline bool Jugador::jRobot() const { return robot; }

#endif
