#ifndef _PARTIDA_H_
#define _PARTIDA_H_

#include <vector>
#include "tablero.h"
#include "cronometro.h"
#include "fuente.h"
#include "sonido.h"

using namespace std;

/** Clase Partida. Clase para el manejo y funcionamiento de la partida de FreeRummi.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Partida
{
 public:

  // Constructor

  /** Constructor de la clase Partida.
   * @param numJug Indica el número de jugadores que participarán.
   */
  Partida(unsigned numJug = 2);

  // Modificadores

  /** Método encargada de inicializar la partida y el bucle del juego.
   * @param v  Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @param tipoPartida 0 si es una partida simple y -1 si es un concurso. Por defecto es 0.
   */
  void jugar(Video *v, int tipoPartida = 0);

  /** Método encargada de añadir jugadores a la partida.
   * @param j Puntero a un objeto de la clase Jugador.
   * @return Devuelve si el jugador se ha incluido en la partida o si por el contrario ha ocurrido algún error.
   */
  bool anadirJugador(Jugador* j);

  /** Método encargada de eliminar jugadores de la partida.
   * @param j Puntero a un objeto de la clase jugador.
   * @return Devuelve si el jugador se ha eliminado de la partida o si por el contrario ha ocurrido algún error.
   */
  bool eliminarJugador(Jugador* j);

  /** Método encargada de inicializar los soportes de los jugadores.
   */
  void iniciarJugadores();
  
  /** Destructor de la clase Partida.
   */
  ~Partida();

 private:
  // Atributos principales: Tablero, cronometro, monton, jugadores
  Tablero tablero, tableroCopia;
  Cronometro crono;
  Monton monton;
  typedef vector<Jugador *> Jugadores;
  Jugadores jugadores;
  Soporte soporteCopia;

  // Atributos de imagen
  Imagen *fondo, *fMenu, *iGan, *iAba, *suena, *nosuena, *confirmar, *cancelar;

  // Atributos auxiliares
  unsigned numJugadores;
  unsigned jugadorActual;
  Puntero pPulsar, pSoltar, pMovimiento;
  SDL_Event evento;
  Ficha f;
  bool robar, comprobar, salir;
  int turn, cont;
  bool pausa;

  // Atributos para textos
  Fuente cron, info, cambio;

  // Atributos para sonidos
  Efecto correcto, incorrecto, reloj, montonfichas, coger, soltar;

  // Banda sonora
  Musica musica;

  // Funciones privadas

  /** Método encargada de distribuir los eventos según el tipo y de hacer todas las comprobaciones pertinentes durante el turno de un jugador.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @return Devuelve 1 si el jugador se ha quedado sin fichas, -1 si el jugador ha abandonado la partida y 0 si sigue jugando.
   */
  int turno(Video *v);

  /** Método encargada de seleccionar el jugador que entra en juego.
   */
  void incrementarJugadorActual();

  /** Método encargada de mostrar todos los textos que aparecen en la partida: información del jugador y cronometro.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void mostrarTextos(Video *v);

  /** Método para mostrar la pantalla de la partida.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void actualizarPantalla(Video *v);

  /** Método para mostrar la pantalla de cambio de jugador.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void pantallaCambio(Video *v);

  /** Método para mostrar la pantalla del jugador ganador.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void pantallaGanador(Video* v);

  /** Método para mostrar la pantalla del jugador que ha abandonado.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void pantallaAbandonado(Video* v);

  /** Método para confirmar el abandono de la partida.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @return Devuelve si el jugador abandona la partida o no.
   */
  bool salirPartida(Video *v);

  /** Método para mostrar la pantalla de confirmación de abandono de la partida.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void pantallaSalir(Video *v);

  // Manejadores de eventos

  /** Método encargado de manejar todos los eventos de pulsación del ratón.
   * @return Devuelve si el evento en cuestión hacer finalizar el turno del jugador.
   */
  bool eventosPulsacion();

  /** Método encargado de manejar todos los eventos de liberación del ratón.
   * @return Devuelve si el evento en cuestión hacer finalizar el turno del jugador.
   */
  bool eventosLiberacion();

  /** Método encargado de manejar todos los eventos de teclado.
   * @return Devuelve si el evento en cuestión hacer finalizar el turno del jugador.
   */
  bool eventosTeclado();
};

#endif

