#ifndef _SOPORTE_H_
#define _SOPORTE_H_

#include <vector>
#include "ficha.h"
#include "monton.h"

/** Clase Soporte. Clase para el manejo y funcionamiento del soporte de fichas del juego.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Soporte {
 public:

  // Constructor

  /** Constructor de la clase Soporte. Inicializa el vector de fichas con fichas nulas.
   */
  Soporte();

  /** Constructor de copia de la clase Soporte.
   * @param s Referencia constante a un objeto de la clase Soporte.
   */
  Soporte(const Soporte& s);
 
  // Sobrecarga del operador de asignacion

  /** Operador de asignación de la clase Soporte.
   * @param s Referencia constante a un objeto de la clase Soporte.
   */
  Soporte& operator =(const Soporte& s);

  // Modificadores

  /** Método que extrae una ficha del soporte.
   * @param pos Indica la posición del soporte donde se encuentra la ficha que se desea extraer.
   * @return Devuelve una referencia constante a un objeto de la clase Ficha. Si en la posición indicada no hay ficha se devolverá una ficha nula.
   */
  const Ficha& escogerFicha(int pos);

  /** Método que inserta una ficha en el soporte.
   * @param f Referencia constante a un objeto de la clase Ficha. Es la ficha que se desea insertar.
   * @param pos Indica la posición del soporte en la que se desea insertar la ficha.
   * @return Devuelve si la ficha se pudo insertar o no.
   */
  bool ordenar(const Ficha& f, int pos);

  /** Método encargado de insertar una ficha robada del montón.
   * @param m Referencia a un objeto de la clase Monton.
   */
  void robar(Monton& m);

  /** Método encargado de actualizar el atributo que informa si el soporte está abierto o no. Para abrir un soporte, es necesario extraer de el 30 o más puntos en un mismo turno. Si un soporte no está abierto y la suma de las fichas que saca al tablero es menor que 30, el jugador no podrá poner y las fichas esrán devueltas al soporte.
   */
  void abrir();

  /** Método para ordenar el soporte por el valor numerico de las fichas.
   */
  void ordenarNumero();

  /** Método para ordenar el soporte por el color de las fichas. Además ordena por el valor numérico cada color de menor a mayor.
   */
  void ordenarColor();

  // Observadores

  /** Método encargado de mostrar por pantalla el soporte.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void mostrarSoporte(Video *v);

  /** Método que devuelve si el soporte esa vacio o no (no quedan fichas en el).
   * @return Devuelve si el soporte está vacio.
   */
  bool vacio() const;

  /** Método que devuelve el número de fichas restantes en el soporte.
   * @return Devuelve el número de fichas restantes en el soporte.
   */
  const int fichas() const;

  /** Método que devuelve la suma de los valores numéricos de las fichas del soporte.
   * @return Devuelve la suma de los valores numéricos de las fichas del soporte.
   */
  const int sumaSoporte() const;

  /** Método que devuelve si un soporte está abierto o no.
   * @return Devuelve si un soporte está abierto o no.
   */
  bool abierto() const;

  /** Destructor de la clase Soporte.
   */
  ~Soporte();

 private:
  int fichRest;
  int suma;
  Ficha sigExt;
  Ficha sop[28];
  bool abrto;

  void juntarFichas();
  void intercambiar(const int pos1, const int pos2);
};

// Funciones externas

/** Función que devuelve la posición dentro del soporte a partir de las coordenadas del puntero.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve la posición dentro del soporte a partir de las coordenadas del puntero. 
 */
int posSoporte(const Puntero& p);

/** Función que devuelve si las coordenadas de un puntero están dentro del area del soporte o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del soporte.
 */
bool areaSoporte(const Puntero& p);

/** Función que devuelve si las coordenadas de un puntero están dentro del area del botón de ordenar por color o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del botón de ordenar por color.
 */
bool areaColor(const Puntero& p);

/** Función que devuelve si las coordenadas de un puntero están dentro del area del botón de ordenar por número o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del botón de ordenar por número.
 */
bool areaNumero(const Puntero& p);

inline void Soporte::abrir() { abrto = true; }
inline bool Soporte::vacio() const { return fichRest == 0; }
inline const int Soporte::fichas() const { return fichRest; }
inline const int Soporte::sumaSoporte() const { return suma; }
inline bool Soporte::abierto() const { return abrto; }

#endif
